#include	"CLifeAnimation.h"


CLifeAnimation::CLifeAnimation(
	long	inWidth,
	long	inHeight )
:	mBufferHeight(inHeight + 2),
	mBufferWidth(inWidth + 2)
{
		// Allocate two buffers to hold two generations.
	for ( int i = 0; i < 2; i++ )
	{
		if ( !( mLifeBuffer[i] = new char[mBufferWidth * mBufferHeight] ) )
			/* ERROR */;

//		printf( "Life: allocated %dx%d buffer #%d.\n",
//			mBufferWidth, mBufferHeight, i );

			// Erase all, including edges
		char* buffer = mLifeBuffer[i];
		for ( int i = 0; i < mBufferWidth * mBufferHeight; i++ )
			buffer[i] = 0;
	}

		// Reset to random starting state
	Reset( );
}

void CLifeAnimation::Reset( )
{
		// Choose buffer 0 as our first buffer
	mCurrentBuffer = 0;

		// Seed starting buffer randomly.
		// We leave a blank border around the
		// outside which will never change.
	char* buffer = mLifeBuffer[mCurrentBuffer];
	long width = mBufferWidth;
	long height = mBufferHeight;	
	for ( int x = 1; x < width - 1; x++ )
		for ( int y = 1; y < height - 1; y++ )
			buffer[x + width * y] = ((rand( ) % 100) < 40);

		// Clear our generation history. We use an array of
		// hashes to detect short-cycle repeating boards.
	mFrameNumber = 0;
	for ( int i = 0; i < LOOKBACK; i++ )
		mLookbackBuffer[i] = 0;
	
		// Prepare edge wrapping
	WrapEdges( );	
}

CLifeAnimation::~CLifeAnimation( )
{
	delete [] (mLifeBuffer[0]);
	delete [] (mLifeBuffer[1]);
}

void CLifeAnimation::AdvanceOneFrame( )
{
	mFrameNumber++;

	long width		= mBufferWidth;
	long height		= mBufferHeight;

	int bufferIndex = mCurrentBuffer;
	char* oldBuffer	= mLifeBuffer[bufferIndex];
	++bufferIndex %= 2;
	char* newBuffer = mLifeBuffer[bufferIndex];
	mCurrentBuffer = bufferIndex;
	
		// This needs a serious speed tweak...
	int liveNeighbors;
	ulong hash = 0;
	for ( int y = 1; y < height - 1; y++ )
	{
		int w_y = width * y;
		for ( int x = 1; x < width - 1; x++ )
		{
			liveNeighbors = 
				(oldBuffer[ x-1	+ w_y - width ] != 0) +
				(oldBuffer[ x	+ w_y - width ] != 0) +
				(oldBuffer[ x+1	+ w_y - width ] != 0) +
				
				(oldBuffer[ x-1	+ w_y ] != 0) +
				(oldBuffer[ x+1	+ w_y ] != 0) +
				
				(oldBuffer[ x-1	+ w_y + width ] != 0) +
				(oldBuffer[ x	+ w_y + width ] != 0) +
				(oldBuffer[ x+1	+ w_y + width ] != 0);
			
				// Will this cell be alive?
			bool live = ( (liveNeighbors == 2
				&& oldBuffer[x + w_y]) || liveNeighbors == 3 );
				
				// What color should this cell be?
			char newVal = 0;
			if ( live )
			{
				char oldVal = oldBuffer[x + w_y];
				if ( oldVal < COLOR_MAP_SIZE - 1 )
					newVal = oldVal + 1;
				else
					newVal = oldVal;
			}
			
			newBuffer[x + w_y] = newVal;

			if ( newVal )
				hash ^= x ^ (y << 8); 
		}
		
		hash = (hash << 7) | (hash >> 25);
	}
	
		// Prepare edge wrapping for next step
	WrapEdges( );
	
		// Check for a static pattern
	bool repeat = FALSE;
	for ( int i = 0; i < LOOKBACK; i++ )
		if ( hash == mLookbackBuffer[i] )
			repeat = TRUE;
	
	if ( repeat )
		Reset( );
	else
		mLookbackBuffer[mFrameNumber % LOOKBACK] = hash;
}

void CLifeAnimation::RenderBitmap(
	uchar*	inBits,
	long	inBitsLength,
	long	inRowBytes,
	long	inBlockSize,
	unsigned long inColorMap[COLOR_MAP_SIZE] )
{
	//memset( inBits, ((char*) inColorMap)[0], inBitsLength );

	long blockPixels = inBlockSize * sizeof(long);

	char* buffer = mLifeBuffer[mCurrentBuffer];
	long width = mBufferWidth;
	long height = mBufferHeight;
	for ( int y = 1; y < height - 1; y++ )
	{
		long left	= 0;
		long right	= blockPixels - 1;
		long top	= (y - 1) * blockPixels;
		long bottom	= top + blockPixels - 1;

		for ( int x = 1; x < width - 1; x++ )
		{
				// Somewhat fast rectangle drawing loop
			char cellVal = buffer[x + width * y];
			for (
				long y1 = top, y_w = y1 * inRowBytes;
				y1 <= bottom;
				y1++, y_w += inRowBytes )
			{
				*(long*) &inBits[left + y_w] = inColorMap[cellVal];
			}
			
			right += blockPixels;
			left += blockPixels;
		}
	}
}

void CLifeAnimation::WrapEdges( )
{
	char* buffer = mLifeBuffer[mCurrentBuffer];
	int width = mBufferWidth;
	int height = mBufferHeight;
	
		// Copy the top and bottom for vertical wrapping
	int real_bottom = width * (height - 2);
	int wrap_bottom = width * (height - 1);
	for ( int i = 1; i < width - 1; i++ )
	{
		buffer[i] = buffer[i + real_bottom];
		buffer[i + wrap_bottom] = buffer[i + width];
	}
	
		// Copy the left and right edges for horizontal wrapping
	for ( int i = 1, pos = 0; i < height - 1;
		i++, pos += width )
	{
		buffer[pos] = buffer[pos + (width - 2)];
		buffer[pos + (width - 1)] = buffer[pos + 1];
	}
	
		// Wrap the four corners
	 buffer		[1 				+ width * 1]
	 	= buffer[(width - 1)	+ width * (height - 1)];
	 buffer		[(width - 2)	+ width * 1]
	 	= buffer[0				+ width * (height - 1)];
	 buffer		[1				+ width * (height - 2)]
	 	= buffer[(width - 1) 	+ width * 0];
	 buffer		[(width - 2)	+ width * (height - 2)]
	 	= buffer[0	 			+ width * 0];
}
