// ============================================================
//  CLifeSaveThread.cpp                Copyright 1996 Eric Kidd
// ============================================================

#include	"CLifeSaveThread.h"
#include	"CLifeAnimation.h"

# define	CELL_SIZE	(4)
# define	SNOOZE_TIME	(200000.0)

unsigned long kDefaultColorMap[COLOR_MAP_SIZE] = {
	0x00000000 /* Black */,
	0xDFDFDFDF /* Dark green */,
	0x2A2A2A2A /* Red */,
	0xFAFAFAFA /* Yellow */,
	0x20202020 /* Blue */
};	

CLifeSaveThread *gSaveThread = NULL;

void module_initialize( void *inSettings, long inSettingsSize )
{
}

void module_cleanup( void **outSettings, long *outSettingsSize )
{
	*outSettings = NULL;
	*outSettingsSize = 0;
}

void module_start_saving( BView *inView )
{	
	inView->Window( )->Show( );
	gSaveThread = new CLifeSaveThread;
	gSaveThread->StartSaving( inView );	
}

void module_stop_saving( )
{
	gSaveThread->StopSaving( );
	delete gSaveThread;
	gSaveThread = NULL;
}

void module_start_config( BView *inView )
{
	if ( !inView->Window( )->Lock( ) )
		return;

	BStringView *view = NULL;
	
	view = new BStringView( BRect( 20.0, 20.0, 100.0, 32.0 ),
		B_EMPTY_STRING, "Life" );
		
	inView->AddChild( view );
	view->SetViewColor( inView->ViewColor( ) );
	view->SetFontName( "Emily" );
	
	view = new BStringView( BRect( 30.0, 40.0, 230.0, 52.0 ),
		B_EMPTY_STRING, "Copyright  1996 Eric Kidd" );

	inView->AddChild( view );
	view->SetViewColor( inView->ViewColor( ) );
	
	inView->Window( )->Unlock( );
}

void module_stop_config( )
{
}


CLifeSaveThread::CLifeSaveThread( )
{
}

void CLifeSaveThread::StartSaving( BView* view )
{
	view->Window( )->Lock( );
	BRect bounds = view->Bounds( );
	bounds.OffsetTo( B_ORIGIN );

		// Calculate buffer dimensions	
		// XXX - What about fractional rounding?
	long pixelWidth		= bounds.right;
	long pixelHeight	= bounds.bottom;
	long bufferWidth	= pixelWidth / CELL_SIZE;
	long bufferHeight	= pixelHeight / CELL_SIZE;
	
		// Allocate buffers
	mLifeAnimation = new CLifeAnimation( bufferWidth, bufferHeight );
	if ( !mLifeAnimation )
		/* ERROR */;
	
	mBitmap = new BBitmap( BRect( 0, 0, pixelWidth, pixelHeight ),
		B_COLOR_8_BIT );
	if ( !mBitmap )
		/* ERROR */;
	memset( mBitmap->Bits( ), ((char*) kDefaultColorMap)[0],
		mBitmap->BitsLength( ) );

	view->Window( )->Unlock( );
	CSaveThread::StartSaving( view );
}

void CLifeSaveThread::StopSaving( )
{
	CSaveThread::StopSaving( );
	
	delete mLifeAnimation;
	if ( mView->Window( ) )
	{
		mView->Window( )->Lock( );
		mView->Sync( ); // before deleting bitmap!
		mView->Window( )->Unlock( );	
	}
	delete mBitmap;
}

double CLifeSaveThread::Save( )
{
	mView->Window( )->Lock( );
	
	mView->Sync( );
	mLifeAnimation->RenderBitmap(
		(uchar*) mBitmap->Bits( ), mBitmap->BitsLength( ),
		mBitmap->BytesPerRow( ), CELL_SIZE / 4,
		kDefaultColorMap );	
	mView->DrawBitmapAsync( mBitmap );
	mView->Flush( );
	
	mLifeAnimation->AdvanceOneFrame( );	
	mView->Window( )->Unlock( );	
	return 0.0;
}


