# include	"CLifeScreen.h"
# include	"CLifeAnimation.h"

# define	CELL_SIZE			(4)
# define	SCREEN_HORIZONTAL	(640)
# define	SCREEN_VERTICAL		(480)

sem_id CLifeScreen::sStartupSem = B_ERROR;

unsigned long kDefaultColorMap[COLOR_MAP_SIZE] = {
	0x00000000 /* Black */,
	0xDFDFDFDF /* Dark green */,
	0x2A2A2A2A /* Red */,
	0xFAFAFAFA /* Yellow */,
	0x20202020 /* Blue */
};	

void CLifeScreen::Initialize( )
{
		// There's a race condition between the
		// BWindowScreenConstructor and the first
		// call to ScreenConnected in DR8.x.
	sStartupSem = create_sem( 0, "Life Startup" );
}

CLifeScreen::CLifeScreen( )
	: BWindowScreen( "Life screen", B_8_BIT_640x480 )
{
	mDrawSem = create_sem( 0, "Life Direct Draw" );
	if ( mDrawSem < B_NO_ERROR )
		/* ERROR */;
	mRunningSem = create_sem( 1, "Life Running" );
	if ( mRunningSem < B_NO_ERROR )
		/* ERROR */;
	mAnimatorQuitSem = create_sem( 0, "Life Animator Quit" );
	if ( mAnimatorQuitSem < B_NO_ERROR )
		/* ERROR */;
		
	mLifeAnimation = new CLifeAnimation( SCREEN_HORIZONTAL / CELL_SIZE,
		SCREEN_VERTICAL / CELL_SIZE );
	if (!mLifeAnimation)
		/* ERROR */;
	mKeyboardReader = new BView( BRect( 0, 0, 5, 5 ), "KeyRead", 0, 0 );
	if (!mKeyboardReader)
		/* ERROR */;

	thread_id animator = spawn_thread( &thread_entry, "Life Animator",
		B_LOW_PRIORITY, this );
	if ( animator < B_NO_ERROR )
		/* ERROR */;
	resume_thread( animator );
	
	release_sem( sStartupSem );
}

void CLifeScreen::Quit( )
{
		// There are major ordering dependencies here!
	acquire_sem( mRunningSem );			// Stop running
	Disconnect( );						// Stop drawing
	acquire_sem( mAnimatorQuitSem );	// Wait for quit

	delete_sem( mDrawSem );
	delete_sem( mRunningSem );
	delete_sem( mAnimatorQuitSem );
	
	delete mLifeAnimation;
	BWindowScreen::Quit( );
}

void CLifeScreen::ScreenConnected( bool inIsActive )
{
		// XXX - DR8.x bug: wait for constructor to finish
	acquire_sem( sStartupSem );
	release_sem( sStartupSem );
	
	if ( inIsActive )
	{
		short height;
		if ( ProposeFrameBuffer( 8, SCREEN_HORIZONTAL, &height ) != B_NO_ERROR
				|| height < (SCREEN_VERTICAL * 2)
				|| SetFrameBuffer( (SCREEN_VERTICAL * 2), SCREEN_HORIZONTAL,
					SCREEN_VERTICAL ) != B_NO_ERROR )
			PostMessage( B_QUIT_REQUESTED );
		else
		{
			mCardInfo = CardInfo( );
			mFrameBufferInfo = FrameBufferInfo( );
			mCurrentFrame = 0;
			RenderCurrentFrame( );
			release_sem( mDrawSem );
		}
	}
	else
	{
		acquire_sem( mDrawSem );
	}
}

long CLifeScreen::thread_entry( void* inData )
{
	((CLifeScreen*) inData)->ThreadEntry( );
	return 0;
}

void CLifeScreen::ThreadEntry( )
{
	int result = B_NO_ERROR;

	while ( 1 )
	{
		result = acquire_sem_etc( mRunningSem, 1, B_TIMEOUT, 1.0 );
		if ( result != B_NO_ERROR )
			break;

		result = acquire_sem_etc( mDrawSem, 1, B_TIMEOUT, 1000000.0 );
		if ( result == B_NO_ERROR )
		{
			mLifeAnimation->AdvanceOneFrame( );
			//ToggleCurrentFrame( );
			RenderCurrentFrame( );
			//DisplayCurrentFrame( );
			release_sem( mDrawSem );
		}	
		release_sem( mRunningSem );
	}
	
	release_sem( mAnimatorQuitSem );
}

void CLifeScreen::ToggleCurrentFrame( )
{
	mCurrentFrame = (mCurrentFrame + 1) % 2;
}

void CLifeScreen::DisplayCurrentFrame( )
{
	SetDisplayArea( SCREEN_HORIZONTAL, SCREEN_VERTICAL, 0,
		mCurrentFrame * SCREEN_VERTICAL );
}

void CLifeScreen::RenderCurrentFrame( )
{
	int frameLength = mFrameBufferInfo->bytes_per_row * SCREEN_VERTICAL;
	unsigned char* baseAddress = ((unsigned char*) mCardInfo->frame_buffer) +
		mCurrentFrame * frameLength;
	mLifeAnimation->RenderBitmap( baseAddress, frameLength,
		mFrameBufferInfo->bytes_per_row, CELL_SIZE / 4, kDefaultColorMap );
}
