# include	<stdio.h>
# include	"CLifeAnimation.h"

# define	ITERATIONS	(100)

# define	CELL_SIZE			(4)
# define	SCREEN_HORIZONTAL	(640)
# define	SCREEN_VERTICAL		(480)

extern double TestAnimationEngine( );
extern double TestRenderingEngine( );
extern double TestEraser( );
extern double TestWrapper( );

unsigned long kDefaultColorMap[COLOR_MAP_SIZE] = {
	0x00000000 /* Black */,
	0xDFDFDFDF /* Dark green */,
	0x2A2A2A2A /* Red */,
	0xFAFAFAFA /* Yellow */,
	0x20202020 /* Blue */
};	

int main( int argc, char** argv )
{
		// So we can make BBitmap objects...
	BApplication* app = new BApplication( '????' );

	double engineTime = TestAnimationEngine( );
	printf( "Calculate one life frame: %f microseconds\n",
		 engineTime / ITERATIONS );

	double renderTime = TestRenderingEngine( );
	printf( "Render one life frame: %f microseconds\n",
		 renderTime / ITERATIONS );

	double eraseTime = TestEraser( );
	printf( "Erase one life frame: %f microseconds\n",
		 eraseTime / ITERATIONS );

	double wrapTime = TestWrapper( );
	printf( "Wrap one life frame: %f microseconds\n",
		 wrapTime / ITERATIONS );

	delete app;
	return 0;
}



double TestAnimationEngine( )
{
	CLifeAnimation animation( SCREEN_HORIZONTAL / CELL_SIZE,
		SCREEN_VERTICAL / CELL_SIZE );

	double startTime = system_time( );
	for ( int i = 0; i < ITERATIONS; i++ )
		animation.AdvanceOneFrame( );	
	double endTime = system_time( );

	return endTime - startTime;
}

double TestRenderingEngine( )
{
	CLifeAnimation animation( SCREEN_HORIZONTAL / CELL_SIZE,
		SCREEN_VERTICAL / CELL_SIZE );
	BBitmap bitmap( BRect( 0, 0, SCREEN_HORIZONTAL, SCREEN_VERTICAL ),
		B_COLOR_8_BIT );

		// Advance to a typical frame
	for ( int i = 0; i < 100; i++ )
		animation.AdvanceOneFrame( );
	
	double startTime = system_time( );
	for ( int i = 0; i < ITERATIONS; i++ )
		animation.RenderBitmap(
			(uchar*) bitmap.Bits( ), bitmap.BitsLength( ),
			bitmap.BytesPerRow( ), CELL_SIZE / 4,
			kDefaultColorMap );
	double endTime = system_time( );

	return endTime - startTime;
}

double TestEraser( )
{
	BBitmap bitmap( BRect( 0, 0, SCREEN_HORIZONTAL, SCREEN_VERTICAL ),
		B_COLOR_8_BIT );
	long* bits = (long*) bitmap.Bits( );
	long length = bitmap.BitsLength( ) / sizeof(long);
	
	double startTime = system_time( );
	for ( int i = 0; i < ITERATIONS; i++ )
		for ( int j = 0; j < length; j++ )
			bits[j] = kDefaultColorMap[0];
//		memset( (uchar*) bits, ((char*) kDefaultColorMap)[0],
//			bitmap.BitsLength( ) );
	double endTime = system_time( );

	return endTime - startTime;
}

double TestWrapper( )
{
	CLifeAnimation animation( SCREEN_HORIZONTAL / CELL_SIZE,
		SCREEN_VERTICAL / CELL_SIZE );

	double startTime = system_time( );
	for ( int i = 0; i < ITERATIONS; i++ )
		animation.WrapEdges( );	
	double endTime = system_time( );

	return endTime - startTime;
}
